"""
Definition of utilitary routines working on graphs.

Define also the interface with networkx module.

"""

from functools import wraps
import networkx as nx


def dict_to_nx(graph, oriented=False):
    """Return an nx.Graph equivalent of given {node: succs}"""
    nxg = nx.DiGraph() if oriented else nx.Graph()
    for node, succs in graph.items():
        for succ in succs:
            nxg.add_edge(node, succ)
    return nxg


def process_input_graph(func):
    """Decorator, ensuring first argument is a networkx graph object.
    If the first arg is a dict {node: succs}, a networkx graph equivalent
    to the dict will be send in place of it."""
    @wraps(func)
    def wrapped_func(*args, **kwargs):
        input_graph = args[0]
        if isinstance(input_graph, nx.DiGraph):
            return func(*args, **kwargs)
        else:
            nx_graph = dict_to_nx(args[0], oriented=True)
            args = [nx_graph] + list(args[1:])
            return func(*args, **kwargs)
    return wrapped_func


def graphviz_layout(nx_graph, prog):
    """Return the layout computed for given networkx (Di)Graph instance"""
    return nx.nx_pydot.graphviz_layout(nx_graph, prog=prog)



"""
Tergraw usage exemple

"""
import time
import tergraw


GRAPH = {'a': {'b', 'c'}, 'd': {'b', 'e'}, 'f': {'g'}}


for frame in tergraw.pretty_view(GRAPH, oriented=True, construction=True):
    print(chr(27) + "[2J")
    print('\n'.join(frame))
    print('↥↥↥↥↥↥↥↥↥↥↥↥↥↥↥ CONSTRUCTION ↥↥↥↥↥↥↥↥↥↥↥↥↥↥↥↥')
    time.sleep(0.1)

print('↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧ ORIENTED ↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧')
print('\n'.join(tergraw.pretty_view(GRAPH, oriented=True)))

print('↧↧↧↧↧↧↧↧↧↧↧↧↧↧ NON ORIENTED ↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧')
print('\n'.join(tergraw.pretty_view(GRAPH, oriented=False)))


GRAPH_LONG_LABEL = {'aaaa': {'bbbb', 'cccc'},
                    'dddd': {'bbbb', 'eeee'},
                    'ffff': {'gggg'},
                    'hh'  : {'i'}}

print('↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧ LONG LABEL ↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧↧')
print('\n'.join(tergraw.pretty_view(GRAPH_LONG_LABEL, oriented=True)))
